require 'xds_b'
require 'xds_b.codeset'
require 'xds_b.xop'

xds_b.pnr = {}
xds_b.pnr.name = {}
xds_b.pnr.description = {}
xds_b.pnr.author = {}
xds_b.pnr.id = {}

require 'xds_b.pnr.soap'
require 'xds_b.pnr.submissionSet'
require 'xds_b.pnr.documentEntry'

local addElement = node.addElement
local setAttr = node.setAttr
local setInner = node.setInner

function xds_b.pnr.new()
   local pnrDocumentSetXML = [[
   <xdsb:ProvideAndRegisterDocumentSetRequest xmlns:xdsb="urn:ihe:iti:xds-b:2007">
       <lcm:SubmitObjectsRequest xmlns:lcm="urn:oasis:names:tc:ebxml-regrep:xsd:lcm:3.0">
           <rim:RegistryObjectList xmlns:rim="urn:oasis:names:tc:ebxml-regrep:xsd:rim:3.0">
           </rim:RegistryObjectList>
       </lcm:SubmitObjectsRequest>
   </xdsb:ProvideAndRegisterDocumentSetRequest>
   ]]
   
   return xml.parse{data=pnrDocumentSetXML}
end

local NameAddHelp = {
   title="xds_b.pnr.name.add",
   usage="xds_b.pnr.name.add{target=<root tag>, name=<value>}",
   description="This function will add a name construct to the xml document.",
   returns={
      "The node tree presenting the resulting xml construct"
   },
   parameters={
      target="The root xml where the name XML will be inserted",
      name="The name"
   },
   examples={"local NameXML = xds_b.pnr.name.add{target=Root, name='John Doe'}"}
}

function xds_b.pnr.name.add(I)
   ihe.help.check(I, NameAddHelp)
   
   local N = addElement(I.target, 'rim:Name')
   local S = addElement(N, 'rim:LocalizedString')
   setAttr(S, 'value', I.name)
   
   return N
end

ihe.help.set{func=xds_b.pnr.name.add, info=NameAddHelp}

local DescriptionAddHelp = {
   title="xds_b.pnr.description.add",
   usage="xds_b.pnr.description.add{target=<root tag>, description=<value>}",
   description="This function will add a description construct to the xml document.",
   returns={
      "The node tree presenting the resulting xml construct"
   },
   parameters={
      target="The root xml where the description XML will be inserted",
      description="The description"
   },
   examples={"local DescriptionXML = xds_b.pnr.description.add{target=Root, description='Annual checkup'}"}
}

function xds_b.pnr.description.add(I)
   ihe.help.check(I, DescriptionAddHelp)
   
   local D = addElement(I.target, 'rim:Description')
   local S = addElement(D, 'rim:LocalizedString')
   setAttr(S, 'value', I.description)
   
   return D
end

ihe.help.set{func=xds_b.pnr.description.add, info=DescriptionAddHelp}
