xds_b.pnr.code = {}

local addElement = node.addElement
local setAttr = node.setAttr

local CodeAddHelp = {
   title="xds_b.pnr.code.add",
   usage=[[xds_b.pnr.code.add{target=<root tag>, object=<value>, objectId=<value>, 
      nodeRepresentation=<value>, id=<value>, scheme=<value>, name=<value>}]],
   description="This function will populate a code element with the appropriate attributes.",
   returns={
      "The node tree presenting the resulting xml construct"
   },
   parameters={
      target="The root xml where the code XML will be inserted",
      object="The codeset UUID",
      objectId="The Id of the contruct containing the code i.e. the submission set or document entry Id",
      nodeRepresentation="The code",
      id="The identifier associated with the code within the request",
      scheme="The codeset the code belongs to",
      name="The display string associated with the code"
   },
   examples={[[local CodeXML = xds_b.pnr.code.add{target=Root, 
      object='urn:uuid:aa543740-bdda-424e-8c96-df4873be850', objectId='SubmissionSetId', 
      nodeRepresentation='History and Physical', id='id_35', 
      scheme='Connect-a-thon contentTypeCodes', name='History and Physical'}]]}
}

function xds_b.pnr.code.add(I)
   ihe.help.check(I, CodeAddHelp)
   
   local C = addElement(I.target, 'rim:Classification')
   setAttr(C, 'classificationScheme', I.object)
   setAttr(C, 'classifiedObject', I.objectId)
   setAttr(C, 'nodeRepresentation', I.nodeRepresentation)
   setAttr(C, 'objectType', 'urn:oasis:names:tc:ebxml-regrep:ObjectType:RegistryObject:Classification')
   setAttr(C, 'id', I.id)
   xds_b.pnr.slot.add{target=C, name='codingScheme', values={I.scheme}}   
   xds_b.pnr.name.add{target=C, name=I.name}    
   
   return C
end

ihe.help.set{func=xds_b.pnr.code.add, info=CodeAddHelp}

