require 'xds_b.pnr'
require 'xds_b.pnr.slot'
require 'xds_b.pnr.documentEntry'
require 'xds_b.pnr.submissionSet'
require 'xds_b.pnr.soap'
require 'xds_b.help'
require 'xds_b.xop'

xds_b.pnr.simpleapi = {}

local setInner = node.setInner

local function WrapInSoap(Body, URL)
   local S = xds_b.pnr.soap.new{url=URL}
   setInner(S['soapenv:Envelope']['soapenv:Body'], tostring(Body['xdsb:ProvideAndRegisterDocumentSetRequest']))
   
   return S
end

local function AddCodes(I)
   local TCodes = I.codes
   for i = 1, #TCodes do
      local CodeType = xds_b.codeset.getCodeType(TCodes[i].code_type)
      local Code = CodeType.Codes[TCodes[i].name]
      xds_b.pnr.code.add{target=I.target, name=Code.Display, scheme=Code.Scheme, 
         nodeRepresentation=Code.Code, object=CodeType.Scheme, 
         objectId=I.parentId, id='id_'..I.index..(i + 4)}
   end
   
   return I.target
end

local function AddDocumentEntryAuthor(I)
   for i = 1, #I.authors do
      local TA = I.authors[i]
      local A = xds_b.pnr.documentEntry.author.add{target=I.target, 
         documentEntryId=I.documentEntryId, id='id_'..I.index..(i + 3)}
      xds_b.pnr.slot.add{target=A, name='authorPerson', 
         values={'^'..TA.person.last..'^'..TA.person.first..'^'..TA.person.prefix..'^^'}}
      local Institutions = {}
      for j = 1, #TA.institutions do
         table.concat(Institutions, TA.institutions[j].value)
      end
      xds_b.pnr.slot.add{target=A, name='authorInstitution', 
         values=Institutions}
      local Roles = {}
      for j = 1, #TA.roles do
         table.concat(Roles, TA.roles[j].value)
      end
      xds_b.pnr.slot.add{target=A, name='authorRole', values=Roles}
      local Specialties = {}
      for j = 1, #TA.specialities do
         table.concat(Specialties, TA.specialities[j].value)
      end         
      xds_b.pnr.slot.add{target=A, name='authorSpecialty', values=Specialties}  
   end
   
   return I.target
end

local function AddDocumentEntryIds(I)
   local TIds = I.ids
   for i = 1, #TIds do
      xds_b.pnr.documentEntry.id.add{target=I.target, idType=TIds[i].id_type, 
         value=TIds[i].value, documentEntryId=I.documentEntryId, 
         id='id_'..I.index..(i + 1)}
   end
   
   return I.target
end

local function AddDocumentEntry(I)
   local DocumentEntry = I.documentEntry
   local Index = I.index
   
   local DE = xds_b.pnr.documentEntry.add{target=I.target, documentEntryId=DocumentEntry.id, 
      mime=DocumentEntry.mime}
   
   xds_b.pnr.slot.add{target=DE, name='creationTime', values={DocumentEntry.creationTime}}   
   xds_b.pnr.slot.add{target=DE, name='languageCode', values={DocumentEntry.languageCode}}   
   xds_b.pnr.slot.add{target=DE, name='sourcePatientId', values={DocumentEntry.sourcePatientId}}   
   xds_b.pnr.slot.add{target=DE, name='serviceStartTime', values={DocumentEntry.serviceStartTime}}      
   xds_b.pnr.slot.add{target=DE, name='serviceStopTime', values={DocumentEntry.serviceStopTime}}        
   xds_b.pnr.slot.add{target=DE, name='sourcePatientInfo', 
      values={
         'PID-3|'..DocumentEntry.sourcePatientInfo.PID3, 
         'PID-5|'..DocumentEntry.sourcePatientInfo.PID5, 
         'PID-7|'..DocumentEntry.sourcePatientInfo.PID7, 
         'PID-8|'..DocumentEntry.sourcePatientInfo.PID8, 
         'PID-11|'..DocumentEntry.sourcePatientInfo.PID11
      }} 
   xds_b.pnr.name.add{target=DE, name=DocumentEntry.name}
   xds_b.pnr.description.add{target=DE, description=DocumentEntry.description} 
      
   local A = {}
   A.target = DE
   A.documentEntryId = DocumentEntry.id
   A.authors = DocumentEntry.authors
   A.index = Index
   AddDocumentEntryAuthor(A)
   
   local C = {}
   C.target = DE
   C.parentId = DocumentEntry.id
   C.codes = DocumentEntry.codes
   C.index = Index
   AddCodes(C)
   
   local Ids = {}
   Ids.target = DE
   Ids.documentEntryId = DocumentEntry.id   
   Ids.ids = DocumentEntry.ids
   Ids.index = Index
   AddDocumentEntryIds(Ids)
   
   xds_b.pnr.documentEntry.association.add{target=I.target, 
      submissionSetId=I.submissionSetId, documentEntryId=DocumentEntry.id, 
      status=DocumentEntry.association_status, id='id_'..Index..'7'}   
   
   return I.target   
end

local function AddSubmissionSetAuthor(I)
   for i = 1, #I.authors do
      local TA = I.authors[i]
      print(TA)
      local A = xds_b.pnr.submissionSet.author.add{target=I.target, 
         submissionSetId=I.submissionSetId, id='id_'..I.index..(i + 5)}
      xds_b.pnr.slot.add{target=A, name='authorPerson', 
         values={'^'..TA.person.last..'^'..TA.person.first..'^'..TA.person.prefix..'^^'}}
      local Institutions = {}
      for j = 1, #TA.institutions do
         table.concat(Institutions, TA.institutions[j].value)
      end
      xds_b.pnr.slot.add{target=A, name='authorInstitution', 
         values=Institutions}
      local Roles = {}
      for j = 1, #TA.roles do
         table.concat(Roles, TA.roles[j].value)
      end
      xds_b.pnr.slot.add{target=A, name='authorRole', values=Roles}
      local Specialties = {}
      for j = 1, #TA.specialities do
         table.concat(Specialties, TA.specialities[j].value)
      end         
      xds_b.pnr.slot.add{target=A, name='authorSpecialty', values=Specialties}  
   end
   
   return I.target
end

local function AddSubmissionSetId(I)
   local TIds = I.ids
   for i = 1, #TIds do
      xds_b.pnr.submissionSet.id.add{target=I.target, idType=TIds[i].id_type, 
         value=TIds[i].value, submissionSetId=I.submissionSetId, 
         id='id_'..I.index..(i + 1)}
   end
   
   return I.target
end

local function AddSubmissionSet(I)
   local SubmissionSet = I.submissionSet
   local Index = I.index
   
   local SS = xds_b.pnr.submissionSet.add{target=I.target, 
      submissionSetId=SubmissionSet.id}

   xds_b.pnr.slot.add{target=SS, name='submissionTime', values={SubmissionSet.submissionTime}}
   xds_b.pnr.name.add{target=SS, name=SubmissionSet.name}
   xds_b.pnr.description.add{target=SS, description=SubmissionSet.description}
   xds_b.pnr.submissionSet.classification.add{target=SS, 
      submissionSetId=SubmissionSet.id, id='id_'..Index..'1'}     
   
   local A = {}
   A.target = SS
   A.submissionSetId = SubmissionSet.id
   A.authors = SubmissionSet.authors
   A.index = Index
   AddSubmissionSetAuthor(A)
   
   local C = {}
   C.target = SS
   C.parentId = SubmissionSet.id   
   C.codes = SubmissionSet.codes
   C.index = Index
   AddCodes(C)
   
   local Ids = {}
   Ids.target = SS
   Ids.submissionSetId = SubmissionSet.id   
   Ids.ids = SubmissionSet.ids
   Ids.index = Index
   AddSubmissionSetId(Ids)
   
   return I.target   
end

local PNRParseResponseHelp = {
   title="xds_b.pnr.simpleapi.parseResponse",
   usage="xds_b.pnr.simpleapi.parseResponse(RawResponse)",
   description=[[This function will parse the return response from the 
   server.]],
   returns={"The parsed response"},
   parameters={"The raw response from the server"},
   examples={"local Results = xds_b.pnr.simpleapi.parseResponse(Response)"}
}

function xds_b.pnr.simpleapi.parseResponse(I)
   local Response = I.response
   
   local Status, ResponseXML = xds_b.xop.soap.extract{response=Response} 
   if Status then
      local XML = xml.parse{data=ResponseXML}
   
      local Root = XML['soapenv:Envelope']['soapenv:Body']['rs:RegistryResponse']
      local Status = Root.status:nodeValue()
   
      if Status:find('Success') then
         return 'Success'
      else
         local Reason = 
         Root['rs:RegistryErrorList']['rs:RegistryError'].codeContext:nodeValue()
         return 'Failure: '..Reason
      end
   else
      return 'Failure: '..ResponseXML
   end
end

local PNRMakeHelp = {
   title="xds_b.pnr.simpleapi.make",
   usage="xds_b.pnr.simpleapi.make(Model)",
   description=[[This function will build a Provide and Register Document Set request 
   from the inputted canonical model.  This function can be used in place of other API 
   that builds xml element by element i.e. the add API functions.]],
   returns={"The HTTP headers and body of the request"},
   parameters={"The canonical model"},
   examples={"local Headers, Body = xds_b.pnr.simpleapi.make(Model)"}
}

xds_b.help.set{func=xds_b.pnr.simpleapi.parseResponse, info=PNRParseResponseHelp}

function xds_b.pnr.simpleapi.make(T) 
   local Attachments = T.attachments
   
   local PnrXML = xds_b.pnr.new()
   local Root = PnrXML['xdsb:ProvideAndRegisterDocumentSetRequest']
   local OBJList = Root['lcm:SubmitObjectsRequest']['rim:RegistryObjectList']
   
   local S = {}
   S.target = OBJList
   S.index = #Attachments + 1
   S.submissionSet = T.submissionSet
   S.submissionSet.id = T.submissionSet.id
   AddSubmissionSet(S)   
   
   for i = 1, #Attachments do
      local D = {}
      D.target = OBJList
      D.index = i
      D.documentEntry = T.documentEntries[i]
      D.submissionSetId = T.submissionSet.id
      AddDocumentEntry(D)
   end
   
   for i = 1, #Attachments do
      xds_b.xop.documentInclude.add{target=Root, documentEntryId=Attachments[i].id, 
         documentUUID=Attachments[i].uuid, index=i}
   end

   local S = WrapInSoap(PnrXML, T.url)
   
   local HTTPHeaders, ContentUUID, MimeUUID = 
   xds_b.getHTTPHeaders{action='ProvideAndRegisterDocumentSet-b'}     
   local PackagedBody = xds_b.xop.attachments.add{content=ContentUUID, mime=MimeUUID, 
      soap=S, attachments=Attachments}
 
   return HTTPHeaders, PackagedBody
end

xds_b.help.set{func=xds_b.pnr.simpleapi.make, info=PNRMakeHelp}