xds_b.pnr.slot = {}

local addElement = node.addElement
local setAttr = node.setAttr
local setText = node.setText

local SlotAddHelp = {
   title="xds_b.pnr.slot.add",
   usage="xds_b.pnr.slot.add{target=<root tag>, name=<value>, values={values}}",
   description="This function will add a slot construct to the xml document.",
   returns={
      "The node tree presenting the resulting xml construct"
   },
   parameters={
      target="The root xml where the slot XML will be inserted",
      name="The name or kind of slot",
      values="A set of values associated with the name"
   },
   examples={[[local SlotXML = xds_b.pnr.slot.add{target=A, name='authorInstitution', 
      values={'Cleveland Clinic', 'Berea Community'}}]]}
}

function xds_b.pnr.slot.add(I)
   ihe.help.check(I, SlotAddHelp)
   
   local S = addElement(I.target, 'rim:Slot')
   setAttr(S, 'name', I.name)
   local VL = addElement(S, 'rim:ValueList')
   
   for i = 1, #I.values do    
      setText(addElement(VL, 'rim:Value'), I.values[i]) 
   end
   
   return S
end

ihe.help.set{func=xds_b.pnr.slot.add, info=SlotAddHelp}